/*
Copyright 2017 - 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with the License. A copy of the License is located at
    http://aws.amazon.com/apache2.0/
or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and limitations under the License.
*/

const { DynamoDBClient } = require("@aws-sdk/client-dynamodb");
const {
  DeleteCommand,
  DynamoDBDocumentClient,
  GetCommand,
  PutCommand,
  QueryCommand,
  UpdateCommand,
  BatchWriteCommand,
} = require("@aws-sdk/lib-dynamodb");
const awsServerlessExpressMiddleware = require("aws-serverless-express/middleware");
const bodyParser = require("body-parser");
const express = require("express");
const { randomUUID } = require("crypto");
const {
  S3Client,
  PutObjectCommand,
  GetObjectCommand,
} = require("@aws-sdk/client-s3");

const s3 = new S3Client();

const ddbClient = new DynamoDBClient({ region: process.env.TABLE_REGION });
const ddbDocClient = DynamoDBDocumentClient.from(ddbClient);

let tableName = "newContactsDB";
if (process.env.ENV && process.env.ENV !== "NONE") {
  tableName = tableName + "-" + process.env.ENV;
}

const userIdPresent = true; // TODO: update in case is required to use that definition
const partitionKeyName = "userId";
const partitionKeyType = "S";
const sortKeyName = "contactId";
const sortKeyType = "S";
const hasSortKey = sortKeyName !== "";
const path = "/contacts";
const UNAUTH = "UNAUTH";
const sortKeyPath = hasSortKey ? "/:" + sortKeyName : "";

// declare a new express app
const app = express();
app.use(bodyParser.json());
app.use(awsServerlessExpressMiddleware.eventContext());

// Enable CORS for all methods
app.use(function (req, res, next) {
  res.header("Access-Control-Allow-Origin", "*");
  res.header("Access-Control-Allow-Headers", "*");
  next();
});

// convert url string param to expected Type
const convertUrlType = (param, type) => {
  switch (type) {
    case "N":
      return Number.parseInt(param);
    default:
      return param;
  }
};

//Extract user ID from CognitoAuthenticationProvider in request context
const getUserId = (req) => {
  const userId =
    req.apiGateway.event.requestContext.identity.cognitoAuthenticationProvider.split(
      ":CognitoSignIn:"
    )[1];

  return userId;
};

const devBucketName = process.env.DEV_BUCKET_NAME;
const masterBucketName = process.env.MASTER_BUCKET_NAME;

const getBucketName = () => {
  if (process.env.ENV === "dev") {
    return devBucketName;
  } else if (process.env.ENV === "master") {
    return masterBucketName;
  } else {
    return devBucketName;
  }
};

const getImages = async (imageKey) => {
  const bucketName = getBucketName();
  const params = {
    Bucket: bucketName,
    Key: imageKey,
  };

  try {
    const data = await s3.send(new GetObjectCommand(params));

    return new Promise((resolve, reject) => {
      const chunks = [];

      data.Body.on("data", (chunk) => {
        chunks.push(chunk);
      });

      data.Body.on("end", () => {
        const imageBuffer = Buffer.concat(chunks);
        const base64Image = imageBuffer.toString("base64");

        // Extract content type from S3 response
        if (data.ContentType) {
          contentType = data.ContentType;
        } else {
          // If ContentType is not available, you can infer it based on the file extension
          const fileExtension = imageKey.split(".").pop();
          contentType = `image/${fileExtension}`;
        }

        // Construct the data URI with the dynamic content type
        const dataURI = `data:${contentType};base64,${base64Image}`;
        resolve(dataURI);
      });

      data.Body.on("error", (err) => {
        console.error("Error reading image stream:", err);
        reject({
          statusCode: 500,
          body: JSON.stringify({ error: "Error fetching image from S3" }),
        });
      });
    });
  } catch (err) {
    console.error("Error fetching image from S3:", err);
    return {
      statusCode: 500,
      body: JSON.stringify({ error: "Error fetching image from S3" }),
    };
  }
};

/************************************
 * HTTP Get method to list objects *
 ************************************/
app.get(path, async function (req, res) {
  const userId = getUserId(req);

  if (!userId) {
    res.statusCode = 400;
    return res.json({ error: "User ID not found in the request" });
  }

  var params = {
    TableName: tableName,
    KeyConditionExpression: "#userId = :userId",
    ExpressionAttributeNames: {
      "#userId": partitionKeyName,
    },
    ExpressionAttributeValues: {
      ":userId": userId,
    },
    Select: "ALL_ATTRIBUTES",
  };

  try {
    const data = await ddbDocClient.send(new QueryCommand(params));

    // Fetch and attach images to each item
    const itemsWithImages = await Promise.all(
      data.Items.map(async (item) => {
        if (item.image) {
          const imageData = await getImages(item.image);

          return { ...item, image: imageData };
        } else {
          return item;
        }
      })
    );

    res.json(itemsWithImages);
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: "Could not load items: " + err.message });
  }
});

/*****************************************************
 * HTTP Get method to list contacts based on labels *
 *****************************************************/
app.get(path + "/labels", async function (req, res) {
  const userId = getUserId(req);

  if (!userId) {
    res.statusCode = 400;
    return res.json({ error: "User ID not found in the request" });
  }

  const labels = req.query.labels;

  if (!labels) {
    res.statusCode = 400;
    return res.json({ error: "Labels not found in the request" });
  }

  var params = {
    TableName: tableName,
    KeyConditionExpression: "#userId = :userId",
    FilterExpression: "contains(labels, :labels)",
    ExpressionAttributeNames: {
      "#userId": "userId",
    },
    ExpressionAttributeValues: {
      ":userId": userId,
      ":labels": labels,
    },
    Select: "ALL_ATTRIBUTES",
    removeUndefinedValues: true,
  };

  try {
    const data = await ddbDocClient.send(new QueryCommand(params));

    // Fetch and attach images to each item
    const itemsWithImages = await Promise.all(
      data.Items.map(async (item) => {
        if (item.image) {
          const imageData = await getImages(item.image);

          return { ...item, image: imageData };
        } else {
          return item;
        }
      })
    );
    res.json(itemsWithImages);
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: "Could not load items: " + err.message });
  }
});

/********************************************
 * HTTP Get method to list favorites objects *
 ********************************************/

app.get(path + "/favorite", async function (req, res) {
  try {
    const userId = getUserId(req);

    if (!userId) {
      res.statusCode = 400;
      return res.json({ error: "User ID not found in the request" });
    }

    var params = {
      TableName: tableName,
      IndexName: "isFavorite-index",
      KeyConditionExpression: "isFavorite = :isFavorite",
      FilterExpression: "userId = :userId",
      ExpressionAttributeValues: {
        ":userId": userId,
        ":isFavorite": 1,
      },
      Select: "ALL_ATTRIBUTES",
    };

    const data = await ddbDocClient.send(new QueryCommand(params));
    // Fetch and attach images to each item
    const itemsWithImages = await Promise.all(
      data.Items.map(async (item) => {
        if (item.image) {
          const imageData = await getImages(item.image);

          return { ...item, image: imageData };
        } else {
          return item;
        }
      })
    );

    res.json(itemsWithImages);
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: "Could not load items: " + err.message });
  }
});

/*************************
 * HTTP get item count *
 *************************/

app.get(path + "/count-all", async function (req, res) {
  try {
    const userId = getUserId(req);

    if (userIdPresent) {
      req.body["userId"] = userId || UNAUTH;
    }

    var params = {
      TableName: tableName,
      KeyConditionExpression: "#userId = :userId",
      ExpressionAttributeNames: {
        "#userId": partitionKeyName,
      },
      ExpressionAttributeValues: {
        ":userId": userId,
      },
      Select: "COUNT",
    };

    const data = await ddbDocClient.send(new QueryCommand(params));
    res.json(data.Count);
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: "Could not load items: " + err.message });
  }
});

/********************************
 * HTTP get favorite item count *
 ********************************/

app.get(path + "/count-favorite", async function (req, res) {
  try {
    const userId = getUserId(req);

    if (userIdPresent) {
      req.body["userId"] = userId || UNAUTH;
    }

    var params = {
      TableName: tableName,
      IndexName: "isFavorite-index",
      KeyConditionExpression: "isFavorite = :isFavorite ",
      FilterExpression: "userId = :userId",
      ExpressionAttributeValues: {
        ":userId": userId,
        ":isFavorite": 1,
      },
      Select: "COUNT",
    };

    const data = await ddbDocClient.send(new QueryCommand(params));
    res.json(data.Count);
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: "Could not load items: " + err.message });
  }
});

/********************************
 * HTTP get label item count *
 ********************************/
app.get(path + "/count-label", async function (req, res) {
  try {
    const userId = getUserId(req);

    if (userIdPresent) {
      req.body["userId"] = userId || UNAUTH;
    }

    var params = {
      TableName: tableName,
      IndexName: "userId-labels-index",
      KeyConditionExpression: "userId = :userId", // Assuming userId is the partition key of your GSI
      ExpressionAttributeValues: {
        ":userId": userId,
      },
      // Select: "COUNT",
    };

    const data = await ddbDocClient.send(new QueryCommand(params));

    // Initialize labelCounts as an empty object and use reduce for counting
    const labelCounts = data.Items.reduce((counts, item) => {
      const labels = item.labels || [];

      labels.forEach((labelObj) => {
        const label = labelObj.S;

        // Increment the count for the label
        counts[label] = (counts[label] || 0) + 1;
      });

      return counts;
    }, {});

    res.json(labelCounts);

    // res.json(data.Count);
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: "Could not load items: " + err.message });
  }
});

/*****************************************
 * HTTP Get method for get single object *
 *****************************************/

app.get(path + sortKeyPath, async function (req, res) {
  const params = {};
  const userId = getUserId(req);

  if (userIdPresent && req.apiGateway) {
    params[partitionKeyName] = userId || UNAUTH;
  } else {
    params[partitionKeyName] = req.params[partitionKeyName];
    try {
      params[partitionKeyName] = convertUrlType(
        req.params[partitionKeyName],
        partitionKeyType
      );
    } catch (err) {
      res.statusCode = 500;
      res.json({ error: "Wrong column type " + err });
    }
  }
  if (hasSortKey) {
    try {
      params[sortKeyName] = convertUrlType(
        req.params[sortKeyName],
        sortKeyType
      );
    } catch (err) {
      res.statusCode = 500;
      res.json({ error: "Wrong column type " + err });
    }
  }

  let getItemParams = {
    TableName: tableName,
    Key: params,
  };

  try {
    const data = await ddbDocClient.send(new GetCommand(getItemParams));

    if (data.Item) {
      const imageData = await getImages(data.Item.image);
      res.json({ ...data.Item, image: imageData });
    } else {
      res.json(data);
    }
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: "Could not load items: " + err.message });
  }
});

/************************************
 * HTTP put method for insert object *
 *************************************/

app.put(path + "/edit" + sortKeyPath, async function (req, res) {
  try {
    const userId = getUserId(req);
    if (userIdPresent) {
      req.body["userId"] = userId || UNAUTH;
    }

    // Check if there is an image update
    if (req.body.image) {
      // Update the image in S3
      const imageUrl = await uploadImageToS3(userId, req.body.image);
      req.body["image"] = imageUrl;
    }

    // Build the UpdateExpression and ExpressionAttributeValues dynamically
    const updateExpression = [];
    const expressionAttributeNames = {};
    const expressionAttributeValues = {};

    Object.entries(req.body).forEach(([key, value]) => {
      if (key !== partitionKeyName && key !== sortKeyName) {
        const placeholder = `#${key}`;
        updateExpression.push(`${placeholder} = :${key}`);
        expressionAttributeNames[placeholder] = key;
        expressionAttributeValues[`:${key}`] = value;
      }
    });

    // Update item in DynamoDB
    const updateItemParams = {
      TableName: tableName,
      Key: {
        [partitionKeyName]: req.body[partitionKeyName],
        [sortKeyName]: req.body[sortKeyName],
      },
      UpdateExpression: `SET ${updateExpression.join(", ")}`,
      ExpressionAttributeNames: expressionAttributeNames,
      ExpressionAttributeValues: expressionAttributeValues,
    };

    let data = await ddbDocClient.send(new UpdateCommand(updateItemParams));

    res.json({ success: "put call succeed!", url: req.url, data: data });
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: err, url: req.url, body: req.body });
  }
});

/***********************************************
 * HTTP patch method for update isFavorite status *
 ***********************************************/

app.patch(path + "/updateFavorite" + sortKeyPath, async function (req, res) {
  try {
    const userId = getUserId(req);
    if (userIdPresent) {
      req.body["userId"] = userId || UNAUTH;
    }
    const { isFavorite, contactId } = req.body;

    // Update item in DynamoDB
    const updateItemParams = {
      TableName: tableName,
      IndexName: "isFavorite-index",
      Key: {
        [partitionKeyName]: req.body[partitionKeyName],
        [sortKeyName]: contactId,
      },
      ExpressionAttributeNames: {
        "#isFavorite": "isFavorite",
      },
      ExpressionAttributeValues: {
        ":isFavorite": isFavorite,
      },
      UpdateExpression: "SET #isFavorite = :isFavorite",
      ReturnValues: "ALL_NEW",
    };

    const data = await ddbDocClient.send(new UpdateCommand(updateItemParams));

    // Extract only the isFavorite value
    const updatedIsFavorite = data.Attributes
      ? data.Attributes.isFavorite
      : null;

    res.json({
      success: "update call succeed!",
      url: req.url,
      isFavorite: updatedIsFavorite,
    });
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: err, url: req.url, body: req.body });
  }
});

// Function to handle S3 image upload
async function uploadImageToS3(userId, imageFile) {
  const bucketName = getBucketName();
  const imageKey = `images/${userId}/${randomUUID()}.jpeg`;

  // Decode the base64 data
  const newBuffer = Buffer.from(
    imageFile.replace(/^data:.+;base64,/, ""),
    "base64"
  );

  const s3Params = new PutObjectCommand({
    Bucket: bucketName,
    Key: imageKey,
    Body: newBuffer,
  });

  try {
    await s3.send(s3Params);
    return imageKey;
  } catch (err) {
    throw new Error("Error uploading image to S3: " + err.message);
  }
}

/************************************
 * HTTP post method for insert object *
 *************************************/

app.post(path, async function (req, res) {
  const userId = getUserId(req);
  if (userIdPresent) {
    req.body["userId"] = userId || UNAUTH;
  }

  // Handle image upload to S3
  const imageFile = req.body.image;

  let imageUrl;
  if (imageFile) {
    try {
      imageUrl = await uploadImageToS3(userId, imageFile);
    } catch (err) {
      res.statusCode = 500;
      return res.json({ error: err.message });
    }
  }

  const contacts = {
    ...req.body,
    contactId: randomUUID(),
    image: imageUrl,
  };

  let putItemParams = {
    TableName: tableName,
    Item: contacts,
  };

  try {
    let data = await ddbDocClient.send(new PutCommand(putItemParams));
    res.json({ success: "post call succeed!", url: req.url, data: data });
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: err, url: req.url, body: req.body });
  }
});

/**************************************
 * HTTP remove method to batch delete object *
 ***************************************/

app.delete(path + "/batch-remove", async function (req, res) {
  const userId = getUserId(req);

  if (!userId) {
    res.statusCode = 400;
    res.json({ error: "Missing userId" });
    return;
  }

  const contactIdsToDelete = req.body.contactIds;

  if (!contactIdsToDelete || !Array.isArray(contactIdsToDelete)) {
    res.statusCode = 400;
    res.json({
      error: "Invalid or missing contactIds array in the request body",
    });
    return;
  }

  const deleteRequests = contactIdsToDelete.map((contactId) => {
    return {
      DeleteRequest: {
        Key: {
          userId: userId,
          contactId: contactId,
        },
      },
    };
  });

  const params = {
    RequestItems: {
      [tableName]: deleteRequests,
    },
  };

  try {
    let data = await ddbDocClient.send(new BatchWriteCommand(params));
    res.json({ url: req.url, data: data });
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: err, url: req.url });
  }
});

/**************************************
 * HTTP remove method to delete object *
 ***************************************/

app.delete(path + "/remove" + sortKeyPath, async function (req, res) {
  const params = {};
  const userId = getUserId(req);

  if (userIdPresent && req.apiGateway) {
    params[partitionKeyName] = userId || UNAUTH;
  } else {
    params[partitionKeyName] = req.params[partitionKeyName];
    try {
      params[partitionKeyName] = convertUrlType(
        req.params[partitionKeyName],
        partitionKeyType
      );
    } catch (err) {
      res.statusCode = 500;
      res.json({ error: "Wrong column type " + err });
    }
  }
  if (hasSortKey) {
    try {
      params[sortKeyName] = convertUrlType(
        req.params[sortKeyName],
        sortKeyType
      );
    } catch (err) {
      res.statusCode = 500;
      res.json({ error: "Wrong column type " + err });
    }
  }

  let removeItemParams = {
    TableName: tableName,
    Key: params,
  };

  try {
    let data = await ddbDocClient.send(new DeleteCommand(removeItemParams));
    res.json({ url: req.url, data: data });
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: err, url: req.url });
  }
});

app.listen(3000, function () {
  console.log("App started");
});

// Export the app object. When executing the application local this does nothing. However,
// to port it to AWS Lambda we will create a wrapper around that will load the app from
// this file
module.exports = app;
