import { API } from "aws-amplify";
import { FetchBaseQueryError } from "@reduxjs/toolkit/query";

import { Contact, UnsavedContact } from "types/contacts";

export async function createContactApi(
  newContact: UnsavedContact
): Promise<{ data: any } | { error: FetchBaseQueryError }> {
  try {
    const data = await API.post("contactAPI", "/contacts", {
      body: newContact,
    });
    return { data: data };
  } catch (err) {
    return {
      error: err as FetchBaseQueryError,
    };
  }
}

export async function getAllContactApi(): Promise<
  { data: Contact[] } | { error: FetchBaseQueryError }
> {
  try {
    const data = await API.get("contactAPI", "/contacts", {});
    return { data: data };
  } catch (err) {
    return {
      error: err as FetchBaseQueryError,
    };
  }
}

export async function getAllContactsWithSpecificLabelApi(
  labels: string
): Promise<{ data: Contact[] } | { error: FetchBaseQueryError }> {
  try {
    const data = await API.get(
      "contactAPI",
      `/contacts/labels?labels=${encodeURIComponent(labels)}`,
      {}
    );
    return { data: data };
  } catch (err) {
    return {
      error: err as FetchBaseQueryError,
    };
  }
}

export async function getListOfFavoriteContactApi(): Promise<
  { data: Contact[] } | { error: FetchBaseQueryError }
> {
  try {
    const data = await API.get("contactAPI", "/contacts/favorite", {});
    return { data: data };
  } catch (err) {
    return {
      error: err as FetchBaseQueryError,
    };
  }
}

export async function getSingleContactApi(
  contactId: string
): Promise<{ data: Contact } | { error: FetchBaseQueryError }> {
  try {
    const data = await API.get("contactAPI", `/contacts/${contactId}`, {});
    return { data };
  } catch (err) {
    return {
      error: err as FetchBaseQueryError,
    };
  }
}

export async function getTotalCountContactApi(): Promise<
  { data: number } | { error: FetchBaseQueryError }
> {
  try {
    const data = await API.get("contactAPI", "/contacts/count-all", {});
    return { data };
  } catch (err) {
    return {
      error: err as FetchBaseQueryError,
    };
  }
}

export async function getTotalCountForFavoriteContactApi(): Promise<
  { data: number } | { error: FetchBaseQueryError }
> {
  try {
    const data = await API.get("contactAPI", "/contacts/count-favorite", {});
    return { data };
  } catch (err) {
    return {
      error: err as FetchBaseQueryError,
    };
  }
}

export async function getTotalCountForContactLabelApi(): Promise<
  { data: number } | { error: FetchBaseQueryError }
> {
  try {
    const data = await API.get("contactAPI", `/contacts/count-label`, {});
    return { data };
  } catch (err) {
    return {
      error: err as FetchBaseQueryError,
    };
  }
}

export async function updateSingleContactApi(
  contact: Contact
): Promise<{ data: any } | { error: FetchBaseQueryError }> {
  try {
    const data = await API.put(
      "contactAPI",
      `/contacts/edit/${contact.contactId}`,
      {
        body: contact,
      }
    );
    return { data: data };
  } catch (err) {
    return {
      error: err as FetchBaseQueryError,
    };
  }
}

export async function updateIsFavoriteStatusContactApi(
  contact: Contact
): Promise<{ data: any } | { error: FetchBaseQueryError }> {
  try {
    const data = await API.patch(
      "contactAPI",
      `/contacts/updateFavorite/${contact.contactId}`,
      {
        body: {
          isFavorite: contact.isFavorite,
          contactId: contact.contactId,
        },
      }
    );
    return { data: data };
  } catch (err) {
    return {
      error: err as FetchBaseQueryError,
    };
  }
}

export async function deleteSingleContactApi(
  contactId: string
): Promise<{ data: any } | { error: FetchBaseQueryError }> {
  try {
    const data = await API.del(
      "contactAPI",
      `/contacts/remove/${contactId}`,
      {}
    );
    return { data: data };
  } catch (err) {
    return {
      error: err as FetchBaseQueryError,
    };
  }
}

export async function batchDeleteContactApi(
  contactIds: string[]
): Promise<{ data: any } | { error: FetchBaseQueryError }> {
  try {
    const data = await API.del("contactAPI", `/contacts/batch-remove`, {
      body: { contactIds: contactIds },
    });
    return { data: data };
  } catch (err) {
    return {
      error: err as FetchBaseQueryError,
    };
  }
}
