import { useEffect } from "react";
import { useSnackbar } from "notistack";
import { IconButton } from "@mui/material";
import Link from "@mui/material/Link";
import ListItemText from "@mui/material/ListItemText";
import Box from "@mui/material/Box";

import { NavItemProps } from "../type";
import { isExternalLink } from "..";
import { useDeleteSingleLabelMutation } from "api/label/labelApi";
import Iconify from "../../Iconify";
import useConfirmDialog from "hooks/useConfirmDialog";

import { ListItemStyle, ListItemTextStyle, ListItemIconStyle } from "./style";

export function NavItemRoot({
  item,
  isCollapse,
  open = false,
  active,
  onOpen,
}: NavItemProps) {
  const { title, path, icon, info, children, labelId } = item;
  const textToRender = `${title}`;

  const { enqueueSnackbar } = useSnackbar();
  const [getConfirmation, Confirmation] = useConfirmDialog();
  const [
    deleteSingleLabel,
    {
      data: deletedLabel,
      error: errorDeletedLabel,
      isError: isErrorDeletedLabel,
    },
  ] = useDeleteSingleLabelMutation();

  const handleDelete = async () => {
    const isConfirmed = await getConfirmation({
      title: "Delete label",
      contentText: "Are you sure you want to delete this label?",
      confirmLabel: `Delete`,
    });
    if (isConfirmed) deleteSingleLabel(labelId || "");
  };

  useEffect(() => {
    if (deletedLabel) {
      enqueueSnackbar(`You have successfully deleted label `, {
        variant: "success",
      });
    }
  }, [deletedLabel]);

  useEffect(() => {
    if (errorDeletedLabel || isErrorDeletedLabel) {
      enqueueSnackbar("Failed to delete label", { variant: "error" });
    }
  }, [deletedLabel]);

  const renderContent = (
    <>
      {icon && <ListItemIconStyle>{icon}</ListItemIconStyle>}
      <ListItemTextStyle
        disableTypography
        primary={textToRender}
        isCollapse={isCollapse}
        sx={{ fontSize: 14 }}
      />
      {!isCollapse && (
        <>
          {info && info}
          {children && <ArrowIcon open={open} />}
        </>
      )}
    </>
  );

  if (children) {
    return (
      <>
        <ListItemStyle onClick={onOpen} activeRoot={active}>
          {renderContent}
        </ListItemStyle>
      </>
    );
  }

  return isExternalLink(path) ? (
    <>
      <ListItemStyle
        component={Link}
        href={path}
        target="_blank"
        rel="noopener"
      >
        {renderContent}
      </ListItemStyle>
    </>
  ) : (
    <Box
      sx={{
        display: "flex",
        justifyContent: "space-between",
        alignItems: "center",
      }}
    >
      <ListItemStyle to={path} activeRoot={active}>
        {renderContent}
      </ListItemStyle>
      {labelId && (
        <IconButton aria-label="delete" onClick={() => handleDelete()}>
          <Iconify key="delete" icon={"mdi:delete-outline"} />
        </IconButton>
      )}
      <Confirmation />
    </Box>
  );
}

// ----------------------------------------------------------------------

type NavItemSubProps = Omit<NavItemProps, "isCollapse">;

export function NavItemSub({
  item,
  open = false,
  active = false,
  onOpen,
}: NavItemSubProps) {
  const { title, path, info, children } = item;

  const renderContent = (
    <>
      <DotIcon active={active} />
      <ListItemText disableTypography primary={title} />
      {info && info}
      {children && <ArrowIcon open={open} />}
    </>
  );

  if (children) {
    return (
      <ListItemStyle onClick={onOpen} activeSub={active} subItem>
        {renderContent}
      </ListItemStyle>
    );
  }

  return isExternalLink(path) ? (
    <ListItemStyle
      component={Link}
      href={path}
      target="_blank"
      rel="noopener"
      subItem
    >
      {renderContent}
    </ListItemStyle>
  ) : (
    <ListItemStyle to={path} activeSub={active} subItem>
      {renderContent}
    </ListItemStyle>
  );
}

// ----------------------------------------------------------------------

type DotIconProps = {
  active: boolean;
};

export function DotIcon({ active }: DotIconProps) {
  return (
    <ListItemIconStyle>
      <Box
        component="span"
        sx={{
          width: 4,
          height: 4,
          borderRadius: "50%",
          bgcolor: "text.disabled",
          transition: (theme) =>
            theme.transitions.create("transform", {
              duration: theme.transitions.duration.shorter,
            }),
          ...(active && {
            transform: "scale(2)",
            bgcolor: "primary.main",
          }),
        }}
      />
    </ListItemIconStyle>
  );
}

// ----------------------------------------------------------------------

type ArrowIconProps = {
  open: boolean;
};

export function ArrowIcon({ open }: ArrowIconProps) {
  return (
    <Iconify
      icon={open ? "eva:arrow-ios-downward-fill" : "eva:arrow-ios-forward-fill"}
      sx={{ width: 16, height: 16, ml: 1 }}
    />
  );
}
