import { useEffect, useState } from "react";
import { DataGrid } from "@mui/x-data-grid";
import { useLocation, useNavigate } from "react-router-dom";
import { useSnackbar } from "notistack";
import Card from "@mui/material/Card";

import Page from "components/Page";
import DataGridToolbar from "components/DataGridToolbar";
import useConfirmDialog from "hooks/useConfirmDialog";
import firstCharToUpperCase from "utils/firstCharToUpperCase";
import { CONTACTS_ROUTES } from "routes/paths";
import {
  useDeleteSingleContactMutation,
  useGetAllContactsQuery,
  useGetAllContactsWithSpecificLabelQuery,
  useGetListOfFavoriteContactsQuery,
  useUpdateIsFavoriteStatusContactMutation,
  useBatchDeleteContactMutation,
} from "api/contact/contactApi";
import { Contact } from "types/contacts";

import useContactsColumns from "./useContactsColumns";
import { Box, Typography } from "@mui/material";

export default function AllContacts() {
  const navigate = useNavigate();
  const [getConfirmation, Confirmation] = useConfirmDialog();

  const { enqueueSnackbar } = useSnackbar();
  const [pageSize, setPageSize] = useState<number>(5);
  const [selectedIds, setSelectedIds] = useState<string[]>([]);

  const { data: allContactsData, isLoading: allContactsLoading } =
    useGetAllContactsQuery(undefined);

  const { data: favoriteContacts, isLoading: favoritesLoading } =
    useGetListOfFavoriteContactsQuery(undefined);
  const [
    deleteSingleContact,
    {
      data: deletedContacts,
      error: errorDeletedContacts,
      isError: isErrorDeletedContacts,
    },
  ] = useDeleteSingleContactMutation();
  const [
    batchDeleteContact,
    {
      data: batchDeletedContacts,
      error: errorBatchDeletedContacts,
      isError: isErrorBatchDeletedContacts,
    },
  ] = useBatchDeleteContactMutation();
  const [
    updateIsFavoriteStatusContact,
    {
      data: updatedIsFavoriteStatus,
      error: errorUpdatedIsFavoriteStatus,
      isError: isErrorUpdatedIsFavoriteStatus,
    },
  ] = useUpdateIsFavoriteStatusContactMutation();

  const location = useLocation();
  const pathname = location.pathname;

  const pathSegments = pathname.split("/").filter(Boolean);
  const lastParam = firstCharToUpperCase(pathSegments[pathSegments.length - 1]);

  const { data: label, isLoading: labelIsLoading } =
    useGetAllContactsWithSpecificLabelQuery(lastParam);

  const isFavoritesParam = pathname.endsWith("favorites");

  let data: Contact[] | undefined, isLoading: boolean, title: string;

  if (pathname.includes("labels")) {
    data = label;
    isLoading = labelIsLoading;
    title = lastParam;
  } else {
    data = isFavoritesParam ? favoriteContacts : allContactsData;
    isLoading = isFavoritesParam ? favoritesLoading : allContactsLoading;
    title = isFavoritesParam ? "Favorite contacts" : "All contacts";
  }

  const handleBatchDelete = async () => {
    const isConfirmed = await getConfirmation({
      title: "Delete contacts",
      contentText: "Are you sure you want to delete these contacts?",
      confirmLabel: `Delete`,
    });
    if (isConfirmed) batchDeleteContact(selectedIds);
  };

  const handleFavorite = (contact: Contact) => {
    const updatedFavoriteStatus = {
      ...contact,
      isFavorite: contact.isFavorite === 1 ? 0 : 1,
    };

    updateIsFavoriteStatusContact(updatedFavoriteStatus);
  };

  const handleEdit = (_id: string) => {
    navigate(`${CONTACTS_ROUTES.edit}/${_id}`);
  };

  const handleDelete = async (_id: string) => {
    const isConfirmed = await getConfirmation({
      title: "Delete contact",
      contentText: "Are you sure you want to delete this contact?",
      confirmLabel: `Delete`,
    });
    if (isConfirmed) deleteSingleContact(_id);
  };

  const columns = useContactsColumns({
    handleFavorite,
    handleDelete,
    handleEdit,
  });

  useEffect(() => {
    if (batchDeletedContacts) {
      enqueueSnackbar(`You have successfully deleted contacts `, {
        variant: "success",
      });
    }
  }, [batchDeletedContacts]);

  useEffect(() => {
    if (errorBatchDeletedContacts || isErrorBatchDeletedContacts) {
      enqueueSnackbar("Failed to delete contacts", { variant: "error" });
    }
  }, [batchDeletedContacts]);

  useEffect(() => {
    if (deletedContacts) {
      enqueueSnackbar(`You have successfully deleted contact `, {
        variant: "success",
      });
    }
  }, [deletedContacts]);

  useEffect(() => {
    if (errorDeletedContacts || isErrorDeletedContacts) {
      enqueueSnackbar("Failed to delete contact", { variant: "error" });
    }
  }, [deletedContacts]);

  useEffect(() => {
    if (updatedIsFavoriteStatus) {
      if (updatedIsFavoriteStatus?.isFavorite === 1) {
        enqueueSnackbar(
          `You have successfully bookmarked a favorite contact `,
          {
            variant: "success",
          }
        );
      } else {
        enqueueSnackbar(
          `You have successfully unbookmarked a favorite contact `,
          {
            variant: "success",
          }
        );
      }
    }
  }, [updatedIsFavoriteStatus]);

  useEffect(() => {
    if (errorUpdatedIsFavoriteStatus || isErrorUpdatedIsFavoriteStatus) {
      enqueueSnackbar("Failed to mark a favorite contact", {
        variant: "error",
      });
    }
  }, [updatedIsFavoriteStatus]);

  return (
    <Page title={title}>
      <Box sx={{ mx: 5 }}>
        <Typography variant="h4">{title}</Typography>
      </Box>
      <Card sx={{ m: 5 }}>
        <DataGrid
          pageSize={pageSize}
          onPageSizeChange={(newPageSize: number) => setPageSize(newPageSize)}
          rowsPerPageOptions={[5, 10, 20, 50, 100]}
          initialState={{
            sorting: { sortModel: [{ field: "name", sort: "asc" }] },
          }}
          rows={data || []}
          columns={columns}
          loading={isLoading}
          getRowId={(row) => row.contactId}
          components={{ Toolbar: DataGridToolbar }}
          componentsProps={{
            toolbar: {
              numSelected: selectedIds.length,
              onDelete: handleBatchDelete,
            },
          }}
          onSelectionModelChange={(ids) => setSelectedIds(ids as string[])}
          disableExtendRowFullWidth={true}
        />
        <Confirmation />
      </Card>
    </Page>
  );
}
